package cz.frantovo.nekurak.servlet;

import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

/**
 * Obal kolem HTTP odpovědi, který zajistí, že i při přesměrování na JSP přes RequestDispatcher
 * se nastaví správný MIME typ a kódování.
 *
 * Možná by mělo fungovat, ale nefunguje nastavení ve web.xml:
 * <jsp-config>
 * <jsp-property-group>
 * <url-pattern>/</url-pattern>
 * <default-content-type>application/xhtml+xml</default-content-type>
 * </jsp-property-group>
 * </jsp-config>
 *
 * Při přesměrování se tam vždy dostalo text/xml
 * nepomohlo ani:
 * <jsp:directive.page contentType="application/xhtml+xml"/>
 * v JSP stránce (které normálně funguje – viz index.jsp).
 *
 * @author fiki
 */
public class XhtmlOdpověď extends HttpServletResponseWrapper {

	private static final String MIME_TYP = "application/xhtml+xml; charset=utf-8";

	public XhtmlOdpověď(HttpServletResponse odpověď) {
		super(odpověď);
	}

	/**
	 * Vynutí správný typ a kódování
	 * @param type ignoruje se
	 */
	@Override
	public void setContentType(String type) {
		super.setContentType(MIME_TYP);
	}

	/**
	 * @return Vynutí správný typ a kódování
	 */
	@Override
	public String getContentType() {
		return MIME_TYP;
	}
}
